<?php

namespace Drupal\storybook_server\Controller;

use Drupal\Core\Controller\ControllerBase;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\Response;

/**
 * Provides an endpoint for Storybook to query.
 * @see https://github.com/storybookjs/storybook/tree/next/app/server
 */
class ServerEndpointController extends ControllerBase {
  public function render(Request $request) {
    $response = new Response();
    $component = $request->query->get('_storybook');
    $component = preg_replace('/[^[:alnum:]\/_-]/', '', $component);
    $filename = "themes/custom/$component.twig";
    // Check that the filepath isn't trying to break out of themes/custom
    if (strpos(realpath(\Drupal::root() . "/$filename") , \Drupal::root() . "/themes/custom/") !== 0) {
      $response->setStatusCode(500);
    }
    else {
      // Load the Twig theme engine so we can use twig_render_template().
      include_once \Drupal::root() . '/core/themes/engines/twig/twig.engine';
      $markup = (string) twig_render_template($filename, $request->query->all());
      $response->setStatusCode(200);
      $response->setContent($markup);
    }
    return $response;
  }
}
